==> react-app-logic-15.4.0.tgz: package/src/gov/forms/proposals/useFundAccountProposalContent.ts <==
import { useCallback, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';

import { CosmosProposalType, OnomyProposals } from '@onomy/client';
import { Blockchain } from '@onomy/frontend-relay-types';
import {
  useDetailedBalance,
  useFixedAmountInput,
  useOnomyHub,
  useStakingAssets,
} from '@onomy/react-hub';
import { useAsyncStatuses } from '@onomy/react-utils';

// TODO: verify these limits or pull from chain?
const TITLE_MAX_LENGTH = 140;
const DESCRIPTION_MAX_LENGTH = 2500;

export function useFundAccountProposalContent({
  chain,
  // cosmos.distribution.v1beta1.CommunityPoolSpendProposal
  type = OnomyProposals.tree.onomyprotocol.dao.v1.FundAccountProposal,
}: {
  chain: Blockchain;
  type?: CosmosProposalType;
}) {
  const { hub } = useOnomyHub();

  const [{ asset }, assetStatus] = useStakingAssets(chain);
  const [title, setTitle] = useState('');
  const [description, setDescription] = useState('');

  const [recipient, setRecipient] = useState('');
  const [amountFixed, setAmountFixed] = useState('');
  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);

  const inputParams = useMemo(() => {
    return {
      title,
      description,

      recipient,
      amount: [
        {
          denom: asset.id,
          amount: amountFixed,
        },
      ],
    };
  }, [title, description, recipient, asset.id, amountFixed]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
    meta: amount.asset.meta,
  });

  const loadingStatus = useAsyncStatuses([assetStatus]);

  const formErrors = useMemo(() => {
    const params = inputParams;

    const errors: {
      title?: string;
      description?: string;
      recipient?: string;
      amountInput?: string;
    } = {};

    if (!params.title) {
      errors.title = 'Title is required';
    } else if (params.title.length >= TITLE_MAX_LENGTH) {
      errors.title = `Title must be less than ${TITLE_MAX_LENGTH} characters`;
    }

    if (!params.description) {
      errors.description = 'Description is required';
    } else if (params.description.length >= DESCRIPTION_MAX_LENGTH) {
      errors.description = `Description must be less than ${DESCRIPTION_MAX_LENGTH} characters`;
    }

    if (!params.recipient) {
      errors.recipient = 'Recipient is required';
    } else if (
      !hub?.chains.onomy.isAddressValid({
        chain,
        id: params.recipient,
      })
    ) {
      errors.recipient = 'Recipient must be a valid address';
    }

    if (!params.amount[0].amount) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(params.amount[0].amount);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    return Object.keys(errors).length > 0 ? errors : null;
  }, [chain, inputParams, hub?.chains.onomy]);

  const canSubmit = useMemo(() => {
    return !formErrors;
  }, [formErrors]);

  const reset = useCallback(() => {
    setTitle('');
    setDescription('');

    setRecipient('');
    setAmountFixed('');
  }, []);

  const encoded = useMemo(() => {
    if (!canSubmit) return null;
    try {
      return (
        type.encode as typeof OnomyProposals.tree.onomyprotocol.dao.v1.FundAccountProposal.encode
      )(inputParams);
    } catch (e) {
      // eslint-disable-next-line no-console
      console.warn(`Error encoding ${type.typeUrl}`, e);
      return null;
    }
  }, [type, canSubmit, inputParams]);

  return {
    TITLE_MAX_LENGTH,
    DESCRIPTION_MAX_LENGTH,

    loadingStatus,
    assetStatus,
    amountStatus,

    chain,
    type,
    amount,

    title,
    description,
    amountInput,
    recipient,

    formErrors,

    setTitle,
    setDescription,

    setAmountInput,
    setRecipient,

    canSubmit,

    encoded,

    reset,
  };
}

==> react-app-logic-17.2.0.tgz: package/src/gov/forms/proposals/useFundAccountProposalContent.ts <==
import { useCallback, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';

import { CosmosProposalType, OnomyProposals } from '@onomy/client';
import { Blockchain } from '@onomy/frontend-relay-types';
import {
  useDetailedBalance,
  useFixedAmountInput,
  useOnomyHub,
  useStakingAssets,
} from '@onomy/react-hub';
import { useAsyncStatuses } from '@onomy/react-utils';
import { OnomyHubUtils } from '@onomy/hub';

// TODO: verify these limits or pull from chain?
const TITLE_MAX_LENGTH = 140;
const DESCRIPTION_MAX_LENGTH = 2500;

export function useFundAccountProposalContent({
  chain,
  // cosmos.distribution.v1beta1.CommunityPoolSpendProposal
  type = OnomyProposals.tree.onomyprotocol.dao.v1.FundAccountProposal,
}: {
  chain: Blockchain;
  type?: CosmosProposalType;
}) {
  const { hub } = useOnomyHub();

  const [{ asset }, assetStatus] = useStakingAssets(chain);
  const [title, setTitle] = useState('');
  const [description, setDescription] = useState('');

  const [recipient, setRecipient] = useState('');
  const [amountFixed, setAmountFixed] = useState('');
  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);

  const inputParams = useMemo(() => {
    return {
      title,
      description,

      recipient,
      amount: [
        {
          denom: asset.id,
          amount: amountFixed,
        },
      ],
    };
  }, [title, description, recipient, asset.id, amountFixed]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
    meta: amount.asset.meta,
  });

  const loadingStatus = useAsyncStatuses([assetStatus]);

  const formErrors = useMemo(() => {
    const params = inputParams;

    const errors: {
      title?: string;
      description?: string;
      recipient?: string;
      amountInput?: string;
    } = {};

    if (!params.title) {
      errors.title = 'Title is required';
    } else if (params.title.length >= TITLE_MAX_LENGTH) {
      errors.title = `Title must be less than ${TITLE_MAX_LENGTH} characters`;
    }

    if (!params.description) {
      errors.description = 'Description is required';
    } else if (params.description.length >= DESCRIPTION_MAX_LENGTH) {
      errors.description = `Description must be less than ${DESCRIPTION_MAX_LENGTH} characters`;
    }

    const cosmosChain = hub ? OnomyHubUtils.getCosmosChain(hub.chains, chain) : null;

    if (!params.recipient || !cosmosChain) {
      errors.recipient = 'Recipient is required';
    } else if (
      !cosmosChain.isAddressValid({
        chain,
        id: params.recipient,
      })
    ) {
      errors.recipient = 'Recipient must be a valid address';
    }

    if (!params.amount[0].amount) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(params.amount[0].amount);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    return Object.keys(errors).length > 0 ? errors : null;
  }, [chain, inputParams, hub]);

  const canSubmit = useMemo(() => {
    return !formErrors;
  }, [formErrors]);

  const reset = useCallback(() => {
    setTitle('');
    setDescription('');

    setRecipient('');
    setAmountFixed('');
  }, []);

  const encoded = useMemo(() => {
    if (!canSubmit) return null;
    try {
      return (
        type.encode as typeof OnomyProposals.tree.onomyprotocol.dao.v1.FundAccountProposal.encode
      )(inputParams);
    } catch (e) {
      // eslint-disable-next-line no-console
      console.warn(`Error encoding ${type.typeUrl}`, e);
      return null;
    }
  }, [type, canSubmit, inputParams]);

  return {
    TITLE_MAX_LENGTH,
    DESCRIPTION_MAX_LENGTH,

    loadingStatus,
    assetStatus,
    amountStatus,

    chain,
    type,
    amount,

    title,
    description,
    amountInput,
    recipient,

    formErrors,

    setTitle,
    setDescription,

    setAmountInput,
    setRecipient,

    canSubmit,

    encoded,

    reset,
  };
}
